﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_draw_equations
{
    class SigmaEquation : Equation
    {
        // The contents.
        private Equation Contents, Above, Below;

        // Dimensions.
        private const float FootFraction = 0.2f;
        private const float AspectRatio = 0.6666f;

        // Initialize the contents.
        public SigmaEquation(Equation contents, Equation above, Equation below)
        {
            Contents = contents;
            Above = above;
            Below = below;
        }

        // Set font sizes for sub-equations.
        public override void SetFontSizes(float font_size)
        {
            FontSize = font_size;
            Contents.SetFontSizes(font_size);
            Above.SetFontSizes(font_size / 2);
            Below.SetFontSizes(font_size / 2);
        }

        // Return the equation's size.
        public override System.Drawing.SizeF GetSize(Graphics gr, Font font)
        {
            // Get sizes.
            SizeF contents_size, above_size, below_size, our_size;
            float symbol_area_width, symbol_area_height, symbol_width, symbol_height;
            GetSizes(gr, font, out contents_size, out above_size, out below_size,
                out our_size, out symbol_area_width, out symbol_area_height, out symbol_width, out symbol_height);
            return our_size;
        }

        // Get sizes.
        private void GetSizes(Graphics gr, Font font, out SizeF contents_size, out SizeF above_size, out SizeF below_size, out SizeF our_size, out float symbol_area_width, out float symbol_area_height, out float symbol_width, out float symbol_height)
        {
            contents_size = Contents.GetSize(gr, font);
            above_size = Above.GetSize(gr, font);
            below_size = Below.GetSize(gr, font);

            float height = Math.Max(
                1.5f * (above_size.Height + below_size.Height),
                contents_size.Height);
            symbol_height = height - above_size.Height - below_size.Height;
            symbol_width = symbol_height * AspectRatio;

            symbol_area_width = Math.Max(
                Math.Max(above_size.Width, below_size.Width),
                symbol_width);
            symbol_area_height = symbol_height + above_size.Height + below_size.Height;

            float width = contents_size.Width + symbol_area_width;

            our_size = new SizeF(width, height);
        }

        // Draw the equation.
        public override void Draw(Graphics gr, Font font, Pen pen, Brush brush, float x, float y)
        {
            // Get sizes.
            SizeF contents_size, above_size, below_size, our_size;
            float symbol_area_width, symbol_area_height, symbol_width, symbol_height;
            GetSizes(gr, font, out contents_size, out above_size, out below_size,
                out our_size, out symbol_area_width, out symbol_area_height, out symbol_width, out symbol_height);

            // Draw Above.
            float above_x = x + (symbol_area_width - above_size.Width) / 2;
            float above_y = y + (our_size.Height - symbol_area_height) / 2;
            Above.Draw(gr, font, pen, brush, above_x, above_y);

            // Draw the sigma symbol.
            float x1 = x + (symbol_area_width - symbol_width) / 2;
            float x2 = x1 + symbol_width;
            float y1 = above_y + above_size.Height;
            float y2 = y1 + symbol_height / 2;
            float y3 = y1 + symbol_height;
            PointF[] sigma_pts = 
                {
                    new PointF(x2, y1 + symbol_height * FootFraction),
                    new PointF(x2, y1),
                    new PointF(x1, y1),
                    new PointF(x2, y2),
                    new PointF(x1, y3),
                    new PointF(x2, y3),
                    new PointF(x2, y3 - symbol_height * FootFraction),
                };
            gr.DrawLines(pen, sigma_pts);

            // Draw Below.
            float below_x = x + (symbol_area_width - below_size.Width) / 2;
            float below_y = y3;
            Below.Draw(gr, font, pen, brush, below_x, below_y);

            // Draw the contents.
            float contents_x = x + symbol_area_width;
            float contents_y = y + (our_size.Height - contents_size.Height) / 2;
            Contents.Draw(gr, font, pen, brush, contents_x, contents_y);
        }
    }
}
